<?

class TimeObject extends BaseObject {

var $time;

function TimeObject($timestamp = 0) {
parent::BaseObject();
$this->time = $timestamp;
}

function setTimestamp($timestamp) {
$this->time = $timestamp;
}

function getTimestamp() {
return $this->time;
}

function toString() {
return ($this->time)?date("d.m.Y H:i:s",$this->time):'';
}

function getDateFormat() {
return "d.m.Y";
}

function getTimeFormat() {
return "H:i";
}

function getDateTimeFormat() {
return "d.m.Y H:i";
}

function getProperty($key) {
$rs = false;
$key = strtolower($key);
switch ($key) {
case "timestamp":
$rs = new IntObject($this->time);
break;
case "date":
$rs = new StringObject(date("d.m.Y", $this->time));
break;
case "time":
$rs = new StringObject(date("H:i", $this->time));
break;
case "datetime":
$rs = new StringObject(date("d.m.Y H:i", $this->time));
break;
default:
$rs = new NodeException("Unknown Property '$key' for TimeObject.");
}
return $rs;
}

//       
 function translateDayOfWeek($day, $short = true) {
global $DIC;
switch ($day) {
case 0: $name = ($short)?$DIC[1374]:""; break;
case 1: $name = ($short)?$DIC[1375]:""; break;
case 2: $name = ($short)?$DIC[1376]:""; break;
case 3: $name = ($short)?$DIC[1377]:""; break;
case 4: $name = ($short)?$DIC[1378]:""; break;
case 5: $name = ($short)?$DIC[1379]:""; break;
case 6: $name = ($short)?$DIC[1380]:""; break;
}
return $name;
}

function timeSpan( $diff ) {
if ( !is_numeric($diff) ) return FALSE;
$ret = array();
$ret["millis"] = $diff - (int)$diff;
$sec = (int)$diff;
$ret["seconds"] = (int)($sec % 60);
$min = $sec / 60;
$ret["minutes"] = (int)($min % 60);
$hour = $min / 60;
$ret["hours"] = (int)($hour % 24);
$day = $hour / 24;
$ret["days"] = (int)($day % 365);
$year = $day / 365;
$ret["years"] = (int)$year;
return $ret;
}

function leadingZero( $int )
{
if ( $int >= 10 ) return $int;
return "0$int";
}

function formatTimeDiffHours( $diff ) {
$span = TimeObject::timeSpan( $diff );
$hours = $span["years"]*365*24 + $span["days"]*24 + $span["hours"];
$ret = "";
if ( $hours > 0 ) $ret = $hours.":";
else $ret = "00:";
$ret .= TimeObject::leadingZero($span["minutes"]).":".TimeObject::leadingZero($span["seconds"]);
return $ret;
}

}

class IntObject extends BaseObject {

var $int;

function IntObject($int) {
parent::BaseObject();
$this->int = (int) $int;
}

function setValue($int) {
$this->int = (int) $int;
}

function getValue() {
return $this->int;
}

function toString() {
return (string) $this->int;
}

}

/**
 *                                                              
 *  
 **/
class NodeArrayObject 
{
function swap( $arr, $key1, $key2 )
{
$newArray = array();
foreach ( $arr as $aKey=>$aValue )
{
if ( $aKey == $key1 )
$newArray[$key2] = $arr[$key2];
else if ( $aKey == $key2 ) 
$newArray[$key1] = $arr[$key1];
else
$newArray[$aKey] = $aValue;
}
return $newArray;	
}

function replace( $arr, $key, $newValue, $newKey=FALSE )
{
if ( is_false($newKey) )
{
$arr[$key] = $newValue;
return $arr;
}
else
{
$newArray = array();
foreach ( $arr as $aKey=>$aValue )
{
if ( $aKey == $key )
$newArray[$newKey] = $newValue;
else
$newArray[$aKey] = $aValue;
}
return $newArray;	
} 
}

function moveUp( $arr, $key )
{
$prevKey = FALSE;
foreach( $arr as $oKey=>$oValue )
{
if ( $oKey == $key )
{
if ( $prevKey === FALSE ) return FALSE;
return NodeArrayObject::swap($arr, $prevKey, $oKey);
}
$prevKey = $oKey;
}
}

function moveDown( $arr, $key )
{
$prevKey = FALSE;
foreach( $arr as $oKey=>$oValue )
{
if ( $prevKey !== FALSE ) 
{
return NodeArrayObject::swap($arr, $prevKey, $oKey);
}
if ( $oKey == $key )
{
$prevKey = $oKey;
}
}
return FALSE;
}

}

class FileObject extends BaseObject {

var $filename;
var $content;
var $modified;

function FileObject($filename = "") {
$this->setFilename($filename);
$this->content = NULL;
$this->modified = false;
}

function readContent($removeWindowsNL = false) {
///                   
 if (!file_exists($this->filename)) return NULL;
$content = "";
$f = fopen($this->filename, "rb");
$content .= fread($f, filesize($this->filename));
fclose($f);
if ($removeWindowsNL) {
$content = str_replace("\r", "", $content);
}
return $content;
}

function loadContent() {
$this->content = $this->readContent();
$this->modified = false;
return ($this->content !== NULL);
}

function getContent() {
return ($this->content === NULL)?$this->readContent():$this->content;
}

function setContent($content) {
$this->content = $content;
$this->modified = true;
}

function resetContent() {
$this->content = NULL;
$this->modified = false;
}

function exists() {
return file_exists($this->filename);
}

function isEmpty() {
return (($this->exists() && filesize($this->filename) == 0) || $this->content === NULL || $this->content === "");
}

function isModified() {
return $this->modified;
}
function toString() {
//                                        
 return $this->getContent();
}

function getFilename() {
return $this->exists()?$this->filename:"";
}

function setFilename($filename) {
$this->filename = $filename;
$this->content = NULL;
}

function storeTo($filename) {
///                                                                                                
 if ($this->content !== NULL) {
//                                    
 $this->filename = $filename;
return $this->store($this->content);
}
if (!$this->filename || !file_exists($this->filename))
return new NodeException("FileObject::storeTo: ".i18n("Filename {0} is either empty or file does not exist.", 0, $this->filename));
if (!$this->_checkDirectory($filename))
return new NodeException("FileObject::storeTo: ".i18n("Can't create directory for file {0}.", 0, $filename));
$rs = @copy($this->filename, $filename);
if ($rs) {
$this->filename = $filename;
$this->modified = false;
return false;
} else {
return new NodeException("FileObject::storeTo: ".i18n("Can't copy file {0} to {1}.", 0, $this->filename, $filename));
}
}

function moveTo($filename) {
///                                                                                                
 if ($this->content !== NULL) {
//                      
 if ($this->exists()) @unlink($this->filename);

//                                    
 $this->filename = $filename;
return $this->store($this->content);
}
if (!$this->filename || !file_exists($this->filename))
return new NodeException("FileObject::moveTo: ".i18n("Filename {0} is either empty or file does not exist.", 0, $this->filename));
if (!$this->_checkDirectory($filename))
return new NodeException("FileObject::moveTo: ".i18n("Can't create directory for file {0}.", 0, $filename));
$rs = @rename($this->filename, $filename);
if ($rs) {
$this->filename = $filename;
$this->modified = false;
return false;
} else {
return new NodeException("FileObject::moveTo: ".i18n("Can't move file {0} to {1}.", 0, $this->filename, $filename));
}
}

function store($content = NULL) {
if ($content === NULL) $content = $this->getContent();

if (!$this->_checkDirectory($this->filename))
return new NodeException("FileObject::store: ".i18n("Can't create directory for file {0}.", 0, $this->filename));
if (!$f = @fopen($this->filename, "wb")) {
return new NodeException("FileObject::store: ".i18n("Can't open file {0} for writing.", 0, $this->filename));
}
if (!fwrite($f, $content)) {
return new NodeException("FileObject::store: ".i18n("Can't write file {0}.", 0, $this->filename));
}
fclose($f);
$this->content = $content;
$this->modified = false;
return false;
}

function _checkDirectory($filename) {
//                              
 $dir = dirname($filename);
if ("$dir" == "") $dir = ".";
node_func("checkpath");
///                                                                            
 return checkpath($dir);
}
}


?>
