<?

class XNLFunctionParser extends NodeObjectParser {

var $d_requestor;

function XNLFunctionParser($requestor) {
parent::NodeObjectParser();
$this->d_requestor = $requestor;
}

function &getRequestor() {
return $this->d_requestor;
}

function keyExists() {
return $this->d_requestor->functionExists($this->getKeyname());
}

function useCache() {
return $this->d_requestor->funcAllowCache($this->getKeyname());
}

function _parse($input, $props) {

global $DEBUG, $TS;
if ($DEBUG["echo"] && $DEBUG["time"]) {
$_ts1 = microtime();
}

//                    
 $edit = $props["edit"]?true:false;
$requestor =& $this->d_requestor;
$_parser =& $this->getParser();
$evaluate = $_parser->doEvaluate();
if (isset($props["evalalways"])) {
$evaluate = $props["evalalways"];
}
$function = $this->getKeyname();
$ret = array();

//                                 
 if (!$requestor->functionExists($function)) {
return new NodeException(i18n("XNL Function '{0}' does not exist.",0,$function));
}
if ($requestor->isAmbigous($function)) {
return new NodeException(i18n("XNL Function '{0}' is ambigous.",0,$function));
}
if (!$requestor->hasSignature($function)) {
$warn = new EWarning(i18n("XNL Function '{0}' has no signature.",0,$function));
$ret[] = $warn;
} else if (!$requestor->checkSignature($function)) {
return new NodeException(i18n("XNL Function '{0}' has an invalid signature!",0,$function));
}

//                     
 $parser = $requestor->getXNLParser($function, $input);
if ($evaluate) {
$parser->execute();
if ($err_id = $parser->getErrorID()) {
return new NodeException(i18n("XNL Error ({0})",0,$err_id).": ".$parser->getErrorMsg());
} else {
$xnlArr = $parser->getResult();
if (is_array($xnlArr)) {
$ret = array_merge($ret, $xnlArr);
} else {
$ret[] = $xnlArr;
}
$deps = $parser->getDependencies();
reset($deps);
while (list($key,$dep) = each($deps)) {
$this->addDependency($dep["type"],$dep["id"],$dep["event"], $dep["ignore"]);
}
}
} else {
$retCode = "";
if ($edit) {
$retCode .= $parser->stripCodeParams(true);
} else {
$params = $parser->parseFunctionParams(true);
$retCode .= implode("\n", $params);
}
if ($retCode) $ret[] = new StringObject($retCode);
}

if ($DEBUG["echo"] && $DEBUG["time"]) {
$_ts2 = microtime();
$TS["sum_xnl_func"] += calctime($_ts1,$_ts2,12);
$TS["sum_xnl_func_invocations"]++;
}

return $ret;
}

function addParam( $name, $value ) {
$this->d_requestor->addParam( $name, $value );
}

}

class XNLPropertyParser extends NodeObjectParser {

var $objects;

function XNLPropertyParser() {
parent::NodeObjectParser();
$this->objects = array();
}

function keyExists() {
return true;
}

function setObject($keyname, $object) {
$this->objects[strtolower($keyname)] = $object;
}

function getObject($keyname) {
$keyname = strtolower($keyname);
if (isset($this->objects[$keyname])) {
return $this->objects[$keyname];
} else {
return new NodeException("Object for key '$keyname' is unknown.");
}
}

function hasObject($keyname) {
return is_object($this->objects[strtolower($keyname)]);
}

function unsetObject($keyname) {
unset($this->objects[strtolower($keyname)]);
}

function clearObjects() {
$this->objects = array();
}

function _parse($input, $props) {
$ret = false;
$prop = $this->getKeyname();
$object = strtolower($props["object"]);
$parser =& $this->getParser();
if ($prop) {
if (true) { //            
 if (!$object || $object == "this") {
if ($this->hasObject("this")) {
$obj = $this->getObject("this");
} else {
$obj = $this->getBaseObject();
}
} else {
$obj = $this->getObject($object);
}
$ret = resolveProperty($prop, $obj);
} else {
$ret = new StringObject($prop);
}
} else {
$ret = new NodeException("Property is not defined.");
}
return $ret;
}
}

class XNLIfParser extends NodeObjectParser {

var $d_else = "";
var $d_parts;

function XNLIfParser() {
parent::NodeObjectParser();
$this->d_else = "";
}

function checkCodeParts($keynames) {		

$parts = $this->checkCondition($keynames["main"]);

return $parts;
}

function _parse($input, $props) {
$rs = "";
if (in_array("main",$this->d_parts)) {
$rs .= $input;
}
if (in_array("else",$this->d_parts)) {
$rs .= $this->d_else;
}

return new StringObject($rs);
}

function setCodeParts($parts) {
$this->d_parts = $parts;
}

function addCodePart($key, $code) {
if (strtolower($key) == "else") {
$this->d_else = $code;
}
}

function clearCodeParts() {
$this->d_else = "";
}

function checkCondition($props) {
global $FEATURE;
$condition = ($props["cond"])?$props["cond"]:$props["condition"];
if (isset($props["evalalways"]) || $FEATURE["xnl_evalalways"]) {
$evaluate = $props["evalalways"];
} else {
$parser = $this->getParser();
$evaluate = $parser->doEvaluate();
}
$rs = array();
if ($evaluate) {

$isTrue = $this->xnlEval(unhtmlentities($condition));

if ($isTrue) {
$rs[] = "main";
} else {
$rs[] = "else";
}
} else {
$rs = array("main", "else");
}
return $rs;
}

function xnlEval($condition) {
global $FEATURE, $DEBUG;

$ret = false;
if ($FEATURE["force_node2"]) {
if ($DEBUG["xnl"]) echo "XNL cond($con)<br>\n";
eval("\$ret = ($condition);");
} else {
$parser =& $this->getParser();
$ret = xnl_evaluate($condition, FALSE);
}

return $ret;
}

}

class XNLValueParser extends NodeObjectParser {

var $d_values;

function XNLValueParser() {
parent::NodeObjectParser();
$this->d_values = array();
}

function _parse($input, $props) {
$code = "";

//             /          
 $get = !is_null($props["get"]);
$set = !is_null($props["set"]);
$name = $this->getKeyname();

if ($set) $this->d_values[$name] = (is_string($props["set"]) && false)?$props["set"]:$input;
if ($get) $code = $this->d_values[$name];

return new StringObject($code);
}
}

?>
